//
//  SVNPalette.m
//  SVNPalette
//
//  Created by Quentin Carnicelli on Fri Aug 06 2004.
//  Copyright (c) 2004 Quentin D. Carnicelli. All rights reserved.
//

#import "SVNPalette.h"

@interface NSFileManager (SVNPaletteExtensions)
- (BOOL)directoryExistsAtPath:(NSString *) path;
@end

@implementation NSFileManager (SVNPaletteExtensions)

- (BOOL)directoryExistsAtPath:(NSString *)path;
{
    BOOL isDirectory;
	return [self fileExistsAtPath:path isDirectory:&isDirectory] && isDirectory;
}

@end

#pragma mark -

static NSString* _kSVNDirectoryName = @".svn";

@implementation SVNPalette

- (void)_setupPrefs
{
	NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
	BOOL original;

	original = [defaults boolForKey: @"DeleteBackup"];

	if( original )
	{
		int result;
		
		result = NSRunAlertPanel(
			@"SVNPalette Configuration Warning",
			@"It appears that you do not have the 'Create backup file when saving' preference enabled. Without this enabled, SVNPalette will not function.",
			@"Open Preferences",
			@"Cancel",
			nil );
		
		if( result == NSOKButton )
		{
			[[[NSApplication sharedApplication] delegate] performSelector: @selector( showPreferencesPanel: )];
		}
	}
}

- (BOOL)_shouldFixSVNDirectoryPermissions
{
	int version = [[NSUserDefaults standardUserDefaults] integerForKey: @"releaseNoteStamp"];
	
	if( version == 0 ) //Error, don't know whats going on
		return NO;
		
	if( version >= 349 )
		return YES;

	return NO;
}

- (BOOL)_shouldMoveSVNDirectory
{
	NSArray* dirs = [[NSUserDefaults standardUserDefaults] objectForKey: @"VersionControlDirectory"];

	return dirs ? ![dirs containsObject: @".svn"] : YES;
}

- (void)_setupNotifications
{
	if( [self _shouldFixSVNDirectoryPermissions] )
	{
		[[NSNotificationCenter defaultCenter]
			addObserver:self
			selector:	@selector(_didOpenDocument:)
			name:		IBDidOpenDocumentNotification
			object:		nil];
	}

	if( [self _shouldMoveSVNDirectory] )
	{
		[[NSNotificationCenter defaultCenter]
			addObserver:self
			selector:	@selector(_didSaveDocument:)
			name:		IBDidSaveDocumentNotification
			object:		nil];
	}
}

- (id)init
{
	self = [super init];
	
	if( self )
	{
		[self _setupPrefs];
		[self _setupNotifications];
	}
	
	return self;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver: self];

	[super dealloc];
}

- (void)warn: (NSString*)frmt, ...
{
#if DEBUG
	va_list argList;

	if( ![frmt length] )
		return;

	va_start(argList, frmt);
	frmt = [[NSString alloc] initWithFormat: frmt arguments: argList];
	va_end(argList);
	NSLog( @"SVNPalette: Warning: %@", frmt );
	[frmt release];
#endif
}

#pragma mark -

- (void)_fixSVNDirectoryPerms: (id <IBDocuments>)document
{
    NSString *path;
    NSFileManager *manager;
    NSString *svnPath;
	char cmd[1024];

    path = [document documentPath];
    manager = [NSFileManager defaultManager];

    // make sure we can find the destination (new) nib (if not, really strange stuff going on)
    if (![manager directoryExistsAtPath:path])
	{
		[self warn: @"Saved document not found (%@)", path];
		return;
	}

    // try to find the svn directory in the backup (old) nib
    svnPath = [path stringByAppendingPathComponent: _kSVNDirectoryName];
    if (![manager directoryExistsAtPath:svnPath])
	{
		[self warn: @"Could not find SVN directory in document (%@, %@)", svnPath, svnPath];
		return;
	}
	
	snprintf( cmd, sizeof(cmd), "chmod -R u+w \"%s\"", [svnPath fileSystemRepresentation] );
	system( cmd );
}

- (void)_moveSVNDirectoryFromBackup: (id <IBDocuments>)document
{
    NSString *path;
    NSFileManager *manager;
    NSString *svnPath;
    NSString *oldPath;

    path = [document documentPath];
    manager = [NSFileManager defaultManager];

    // make sure we can find the destination (new) nib (if not, really strange stuff going on)
    if (![manager directoryExistsAtPath:path])
	{
		[self warn: @"Saved document not found (%@)", path];
		return;
	}

    // try to find the backup (old) nib
    oldPath = [[[path stringByDeletingPathExtension] stringByAppendingString: @"~"] stringByAppendingPathExtension: [path pathExtension]];
    if (![manager directoryExistsAtPath:oldPath])
	{
		[self warn: @"Could not found backup of document (%@, %@)", oldPath, path];
		return;
	}

    // try to find the svn directory in the backup (old) nib
    svnPath = [oldPath stringByAppendingPathComponent: _kSVNDirectoryName];
    if (![manager directoryExistsAtPath:svnPath])
	{
		[self warn: @"Could not find SVN directory in document backup (%@, %@)", svnPath, oldPath];
		goto cleanup;
	}

    // make sure there isn't already a svn directory in the destination (new) nib
    // (if so, something strange going on - some other svn palette already took charge, maybe?)
    path = [path stringByAppendingPathComponent: _kSVNDirectoryName];
    if ([manager directoryExistsAtPath:path])
	{
		[self warn: @"SVN directory already exists in document"];
		goto cleanup;
	}

    // move path rather than copy it so the old nib doesn't look like something that svn cares about
    if (![manager movePath:svnPath toPath:path handler:nil])
	{
		[self warn: @"Could not move SVN directory from old document to new (%@, %@)", oldPath, path];
		goto cleanup;
	}

cleanup:

	if( [[NSUserDefaults standardUserDefaults] boolForKey: @"SVNPalette.DeleteBackup"] )
	{
		if( ![manager removeFileAtPath: oldPath handler: nil] )
			[self warn: @"Could not remove document backup (%@)", oldPath];
	}
}

- (void)_didOpenDocument: (NSNotification*)note
{
	[self _fixSVNDirectoryPerms: (id <IBDocuments>) [note object]];
}

- (void)_didSaveDocument: (NSNotification*)note
{
	[self _moveSVNDirectoryFromBackup: (id <IBDocuments>) [note object]];
}

@end
